@@ -27,25 +27,28 @@
  *
  * Sept 9, 1998 - fixed loop markers.
  *
+ * Feb. 9, 1999 - Small fix to work with invalid headers that include
+ *   a INST block with markers that equal 0.  It should ingore those.
+ *   Also fix endian problems when ran on Intel machines.  The check
+ *   for endianness was being performed AFTER reading the header instead
+ *   of before reading it.
+ *
  */
 
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
-#include "st.h"
+#include <stdio.h>
 
-#ifdef SYSV
-#include <unistd.h>
-#else
-
-#ifndef SEEK_SET
-#define SEEK_SET 0		/* nasty nasty */
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>	/* For SEEK_* defines if not found in stdio */
 #endif
-#ifndef SEEK_CUR
-#define SEEK_CUR 1		/* nasty nasty */
-#endif /* SEEK_CUR */
 
-#endif /* SYSV */
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
+#include "st.h"
 
 /* Private data used by writer */
 struct aiffpriv {
@@ -59,8 +62,8 @@ void rawwrite(P3(ft_t, LONG *, LONG));
 void write_ieee_extended(P2(ft_t, double));
 double ConvertFromIeeeExtended();
 void ConvertToIeeeExtended(P2(double, char *));
-void textChunk(char **text, char *chunkDescription, ft_t ft);
-void reportInstrument(ft_t ft);
+void textChunk(P3(char **text, char *chunkDescription, ft_t ft));
+void reportInstrument(P1(ft_t ft));
 
 void aiffstartread(ft) 
 ft_t ft;
@@ -75,7 +78,7 @@ ft_t ft;
 	double rate = 0.0;
 	ULONG offset = 0;
 	ULONG blocksize = 0;
-	int littlendian = 0;
+	int littlendian = 1;
 	char *endptr;
 	int foundcomm = 0, foundmark = 0, foundinstr = 0;
 	struct mark {
@@ -91,11 +94,21 @@ ft_t ft;
 	char *copyright;
 	char *nametext;
 
+	/* Needed because of rawread() */
+	rawstartread(ft);
+
+	/* AIFF is in Big Endian format.  Swap whats read in on Little */
+	/* Endian machines.                                            */
+	endptr = (char *) &littlendian;
+	if (*endptr)
+	{
+	    ft->swap = ft->swap ? 0 : 1;
+	}
 
 	/* FORM chunk */
 	if (fread(buf, 1, 4, ft->fp) != 4 || strncmp(buf, "FORM", 4) != 0)
 		fail("AIFF header does not begin with magic word 'FORM'");
-	totalsize = rblong(ft);
+	totalsize = rlong(ft);
 	if (fread(buf, 1, 4, ft->fp) != 4 || strncmp(buf, "AIFF", 4) != 0)
 		fail("AIFF 'FORM' chunk does not specify 'AIFF' as type");
 
@@ -104,27 +117,28 @@ ft_t ft;
 	/* The SSND chunk must be the last in the file */
 	while (1) {
 		if (fread(buf, 1, 4, ft->fp) != 4)
+		{
 			if (ssndsize > 0)
 				break;
 			else
 				fail("Missing SSND chunk in AIFF file");
-
+		}
 		if (strncmp(buf, "COMM", 4) == 0) {
 			/* COMM chunk */
-			chunksize = rblong(ft);
+			chunksize = rlong(ft);
 			if (chunksize != 18)
 				fail("AIFF COMM chunk has bad size");
-			channels = rbshort(ft);
-			frames = rblong(ft);
-			bits = rbshort(ft);
+			channels = rshort(ft);
+			frames = rlong(ft);
+			bits = rshort(ft);
 			rate = read_ieee_extended(ft);
 			foundcomm = 1;
 		}
 		else if (strncmp(buf, "SSND", 4) == 0) {
 			/* SSND chunk */
-			chunksize = rblong(ft);
-			offset = rblong(ft);
-			blocksize = rblong(ft);
+			chunksize = rlong(ft);
+			offset = rlong(ft);
+			blocksize = rlong(ft);
 			chunksize -= 8;
 			ssndsize = chunksize;
 			/* if can't seek, just do sound now */
@@ -136,14 +150,14 @@ ft_t ft;
 		}
 		else if (strncmp(buf, "MARK", 4) == 0) {
 			/* MARK chunk */
-			chunksize = rblong(ft);
-			nmarks = rbshort(ft);
+			chunksize = rlong(ft);
+			nmarks = rshort(ft);
 			chunksize -= 2;
 			for(i = 0; i < nmarks; i++) {
 				int len;
 
-				marks[i].id = rbshort(ft);
-				marks[i].position = rblong(ft);
+				marks[i].id = rshort(ft);
+				marks[i].position = rlong(ft);
 				chunksize -= 6;
 				len = getc(ft->fp);
 				chunksize -= len + 1;
@@ -164,38 +178,46 @@ ft_t ft;
 		}
 		else if (strncmp(buf, "INST", 4) == 0) {
 			/* INST chunk */
-			chunksize = rblong(ft);
+			chunksize = rlong(ft);
 			ft->instr.MIDInote = getc(ft->fp);
 			getc(ft->fp);				/* detune */
 			ft->instr.MIDIlow = getc(ft->fp);
 			ft->instr.MIDIhi = getc(ft->fp);
 			getc(ft->fp);			/* low velocity */
 			getc(ft->fp);			/* hi  velocity */
-			rbshort(ft);				/* gain */
-			ft->loops[0].type = rbshort(ft); /* sustain loop */
-			sustainLoopBegin = rbshort(ft);	 /* begin marker */
-			sustainLoopEnd = rbshort(ft);    /* end marker */
-			ft->loops[1].type = rbshort(ft); /* release loop */
-			releaseLoopBegin = rbshort(ft);  /* begin marker */
-			releaseLoopEnd = rbshort(ft);    /* end marker */
-			foundinstr = 1;
+			rshort(ft);				/* gain */
+			ft->loops[0].type = rshort(ft); /* sustain loop */
+			sustainLoopBegin = rshort(ft);	 /* begin marker */
+			sustainLoopEnd = rshort(ft);    /* end marker */
+			ft->loops[1].type = rshort(ft); /* release loop */
+			releaseLoopBegin = rshort(ft);  /* begin marker */
+			releaseLoopEnd = rshort(ft);    /* end marker */
+
+			/* At least one known program generates an INST */
+			/* block with everything zeroed out (meaning    */
+			/* no Loops used).  In this case it should just */
+			/* be ingorned.				        */
+			if (sustainLoopBegin == 0 && releaseLoopBegin == 0)
+				foundinstr = 0;
+			else
+				foundinstr = 1;
 		}
 		else if (strncmp(buf, "APPL", 4) == 0) {
-			chunksize = rblong(ft);
+			chunksize = rlong(ft);
 			while(chunksize-- > 0)
 				getc(ft->fp);
 		}
 		else if (strncmp(buf, "ALCH", 4) == 0) {
 			/* I think this is bogus and gets grabbed by APPL */
 			/* INST chunk */
-			rblong(ft);		/* ENVS - jeez! */
-			chunksize = rblong(ft);
+			rlong(ft);		/* ENVS - jeez! */
+			chunksize = rlong(ft);
 			while(chunksize-- > 0)
 				getc(ft->fp);
 		}
 		else if (strncmp(buf, "ANNO", 4) == 0) {
 			/* Old form of comment chunk */
-			chunksize = rblong(ft);
+			chunksize = rlong(ft);
 			/* allocate enough memory to hold the comment */
 			ft->comment = (char *) malloc((size_t) chunksize);
 			if (ft->comment == NULL)
@@ -229,7 +251,7 @@ ft_t ft;
 			if (feof(ft->fp))
 				break;
 			report("AIFFstartread: ignoring '%s' chunk\n", buf);
-			chunksize = rblong(ft);
+			chunksize = rlong(ft);
 			if (feof(ft->fp))
 				break;
 			/* Skip the chunk using getc() so we may read
@@ -248,11 +270,12 @@ ft_t ft;
 	 * Like, say, instrument loops. 
 	 */
 	if (ft->seekable)
+	{
 		if (seekto > 0)
 			fseek(ft->fp, seekto, SEEK_SET);
 		else
 			fail("AIFF: no sound data on input file");
-
+	}
 	/* SSND chunk just read */
 	if (blocksize != 0)
 		fail("AIFF header specifies nonzero blocksize?!?!");
@@ -290,7 +313,6 @@ ft_t ft;
 
 	p->nsamples = ssndsize / ft->info.size;	/* leave out channels */
 
-	/* process instrument and marker notations. */
 	if (foundmark && !foundinstr)
 		fail("Bogus AIFF file: MARKers but no INSTrument.");
 	if (!foundmark && foundinstr)
@@ -334,15 +356,11 @@ ft_t ft;
 	}
 	if (verbose)
 	  reportInstrument(ft);
-
-	endptr = (char *) &littlendian;
-	*endptr = 1;
-	if (littlendian == 1)
-		ft->swap = 1;
 }
 
 /* print out the MIDI key allocations, loop points, directions etc */
-void reportInstrument(ft_t ft)
+void reportInstrument(ft)
+ft_t ft;
 {
   int loopNum;
 
@@ -368,8 +386,12 @@ void reportInstrument(ft_t ft)
 }
 
 /* Process a text chunk, allocate memory, display it if verbose and return */
-void textChunk(char **text, char *chunkDescription, ft_t ft) {
-  LONG chunksize = rblong(ft);
+void textChunk(text, chunkDescription, ft) 
+char **text;
+char *chunkDescription;
+ft_t ft;
+{
+  LONG chunksize = rlong(ft);
   /* allocate enough memory to hold the text including a terminating \0 */
   *text = (char *) malloc((size_t) chunksize + 1);
   if (*text == NULL)
@@ -403,22 +425,31 @@ ft_t ft;
 	ULONG chunksize;
 
 	if (!ft->seekable)
-	    while (! feof(ft->fp)) {
+	{
+	    while (! feof(ft->fp)) 
+	    {
 		if (fread(buf, 1, 4, ft->fp) != 4)
-			return;
+			break;
 
-		chunksize = rblong(ft);
+		chunksize = rlong(ft);
 		if (feof(ft->fp))
-			return;
+			break;
 		buf[4] = '\0';
 		warn("Ignoring AIFF tail chunk: '%s', %d bytes long\n", 
 			buf, chunksize);
 		if (! strcmp(buf, "MARK") || ! strcmp(buf, "INST"))
 			warn("	You're stripping MIDI/loop info!\n");
 		while ((LONG) (--chunksize) >= 0) 
+		{
 			if (getc(ft->fp) == EOF)
-				return;
+				break;
+		}
+	    }
 	}
+
+	/* Needed because of rawwrite() */
+	rawstopread(ft);
+
 	return;
 }
 
@@ -436,15 +467,28 @@ void aiffstartwrite(ft)
 ft_t ft;
 {
 	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
-	int littlendian = 0;
+	int littlendian;
 	char *endptr;
 
+	/* Needed because rawwrite() */
+	rawstartwrite(ft);
+
+	/* AIFF is in Big Endian format.  Swap whats read in on Little */
+	/* Endian machines.                                            */
+	littlendian = 1;
+	endptr = (char *) &littlendian;
+	if (*endptr)
+	{
+	    ft->swap = ft->swap ? 0 : 1;
+	}
+
 	p->nsamples = 0;
 	if (ft->info.style == ULAW && ft->info.size == BYTE) {
 		report("expanding 8-bit u-law to 16 bits");
 		ft->info.size = WORD;
 	}
 	ft->info.style = SIGN2; /* We have a fixed style */
+
 	/* Compute the "very large number" so that a maximum number
 	   of samples can be transmitted through a pipe without the
 	   risk of causing overflow when calculating the number of bytes.
@@ -452,11 +496,6 @@ ft_t ft;
 	   Sorry, the AIFF format does not provide for an "infinite"
 	   number of samples. */
 	aiffwriteheader(ft, 0x7f000000L / (ft->info.size*ft->info.channels));
-
-	endptr = (char *) &littlendian;
-	*endptr = 1;
-	if (littlendian == 1)
-		ft->swap = 1;
 }
 
 void aiffwrite(ft, buf, len)
@@ -473,6 +512,12 @@ aiffstopwrite(ft)
 ft_t ft;
 {
 	struct aiffpriv *p = (struct aiffpriv *) ft->priv;
+
+	/* Needed because of rawwrite().  Call now to flush
+	 * buffer now before seeking around below.
+	 */
+	rawstopwrite(ft);
+
 	if (!ft->seekable)
 		return;
 	if (fseek(ft->fp, 0L, SEEK_SET) != 0)
@@ -489,6 +534,7 @@ LONG nframes;
 		8 /*SSND hdr*/ + 12 /*SSND chunk*/;
 	int bits = 0;
 	int i;
+	int comment_size;
 
 	hsize += 8 + 2 + 16*ft->instr.nloops;	/* MARK chunk */
 	hsize += 20;				/* INST chunk */
@@ -501,22 +547,31 @@ LONG nframes;
 		fail("unsupported output style/size for AIFF header");
 
 	fputs("FORM", ft->fp); /* IFF header */
-	wblong(ft, hsize + nframes * ft->info.size * ft->info.channels); /* file size */
+	wlong(ft, hsize + nframes * ft->info.size * ft->info.channels); /* file size */
 	fputs("AIFF", ft->fp); /* File type */
 
 	/* ANNO chunk -- holds comments text, however this is */
 	/* discouraged by Apple in preference to a COMT comments */
 	/* chunk, which holds a timestamp and marker id */
-	fputs("ANNO", ft->fp);
-	wblong(ft, (LONG) strlen(ft->comment)); /* ANNO chunk size, the No of chars */
-	fputs(ft->comment, ft->fp);
+	if (ft->comment)
+	{
+	  fputs("ANNO", ft->fp);
+	  /* Must put an even number of characters out.  True 68k processors
+	   * OS's seem to require this 
+	   */
+	  comment_size = strlen(ft->comment);
+	  wlong(ft, (LONG)(((comment_size % 2) == 0) ? comment_size : comment_size + 1)); /* ANNO chunk size, the No of chars */
+	  fputs(ft->comment, ft->fp);
+	  if (comment_size % 2 == 1)
+		fputs(" ", ft->fp);
+	}
 
 	/* COMM chunk -- describes encoding (and #frames) */
 	fputs("COMM", ft->fp);
-	wblong(ft, (LONG) 18); /* COMM chunk size */
-	wbshort(ft, ft->info.channels); /* nchannels */
-	wblong(ft, nframes); /* number of frames */
-	wbshort(ft, bits); /* sample width, in bits */
+	wlong(ft, (LONG) 18); /* COMM chunk size */
+	wshort(ft, ft->info.channels); /* nchannels */
+	wlong(ft, nframes); /* number of frames */
+	wshort(ft, bits); /* sample width, in bits */
 	write_ieee_extended(ft, (double)ft->info.rate);
 
 	/* MARK chunk -- set markers */
@@ -524,22 +579,22 @@ LONG nframes;
 		fputs("MARK", ft->fp);
 		if (ft->instr.nloops > 2)
 			ft->instr.nloops = 2;
-		wblong(ft, 2 + 16*ft->instr.nloops);
-		wbshort(ft, ft->instr.nloops);
+		wlong(ft, 2 + 16*ft->instr.nloops);
+		wshort(ft, ft->instr.nloops);
 
 		for(i = 0; i < ft->instr.nloops; i++) {
-			wbshort(ft, i + 1);
-			wblong(ft, ft->loops[i].start);
+			wshort(ft, i + 1);
+			wlong(ft, ft->loops[i].start);
 			fputc(0, ft->fp);
 			fputc(0, ft->fp);
-			wbshort(ft, i*2 + 1);
-			wblong(ft, ft->loops[i].start + ft->loops[i].length);
+			wshort(ft, i*2 + 1);
+			wlong(ft, ft->loops[i].start + ft->loops[i].length);
 			fputc(0, ft->fp);
 			fputc(0, ft->fp);
 			}
 
 		fputs("INST", ft->fp);
-		wblong(ft, 20);
+		wlong(ft, 20);
 		/* random MIDI shit that we default on */
 		fputc(ft->instr.MIDInote, ft->fp);
 		fputc(0, ft->fp);			/* detune */
@@ -547,30 +602,30 @@ LONG nframes;
 		fputc(ft->instr.MIDIhi, ft->fp);
 		fputc(1, ft->fp);			/* low velocity */
 		fputc(127, ft->fp);			/* hi  velocity */
-		wbshort(ft, 0);				/* gain */
+		wshort(ft, 0);				/* gain */
 
 		/* sustain loop */
-		wbshort(ft, ft->loops[0].type);
-		wbshort(ft, 1);				/* marker 1 */
-		wbshort(ft, 3);				/* marker 3 */
+		wshort(ft, ft->loops[0].type);
+		wshort(ft, 1);				/* marker 1 */
+		wshort(ft, 3);				/* marker 3 */
 		/* release loop, if there */
 		if (ft->instr.nloops == 2) {
-			wbshort(ft, ft->loops[1].type);
-			wbshort(ft, 2);			/* marker 2 */
-			wbshort(ft, 4);			/* marker 4 */
+			wshort(ft, ft->loops[1].type);
+			wshort(ft, 2);			/* marker 2 */
+			wshort(ft, 4);			/* marker 4 */
 		} else {
-			wbshort(ft, 0);			/* no release loop */
-			wbshort(ft, 0);
-			wbshort(ft, 0);
+			wshort(ft, 0);			/* no release loop */
+			wshort(ft, 0);
+			wshort(ft, 0);
 		}
 	}
 
 	/* SSND chunk -- describes data */
 	fputs("SSND", ft->fp);
 	/* chunk size */
-	wblong(ft, 8 + nframes * ft->info.channels * ft->info.size); 
-	wblong(ft, (LONG) 0); /* offset */
-	wblong(ft, (LONG) 0); /* block size */
+	wlong(ft, 8 + nframes * ft->info.channels * ft->info.size); 
+	wlong(ft, (LONG) 0); /* offset */
+	wlong(ft, (LONG) 0); /* block size */
 }
 
 double read_ieee_extended(ft)
