@@ -31,32 +31,33 @@
 #include "patchlvl.h"
 #include <stdio.h>
 #include <string.h>
-#if	defined(unix) || defined(AMIGA) || defined(__OS2__)
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <stdlib.h>
+#include <stdlib.h>		/* for malloc() */
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
 #endif
-#if	defined(__STDC__)
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-#include <ctype.h>
-#ifdef VMS
 #include <errno.h>
+#include <sys/types.h>		/* for fstat() */
+#include <sys/stat.h>		/* for fstat() */
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>		/* for unlink() */
+#endif
+
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#else
+#ifndef HAVE_GETOPT
+int getopt(P3(int,char **,char *));
+#endif
+#endif
+
+#ifdef VMS
 #include <perror.h>
 #define LASTCHAR        ']'
 #else
-#include <errno.h>
 #define LASTCHAR        '/'
 #endif
 
-#ifndef NEED_GETOPT
-#include <getopt.h>
-#endif
-
 /*
  * SOX main program.
  *
@@ -71,10 +72,6 @@
 char amiversion[AmiVerSize]=AmiVerChars;      
 #endif /* AMIGA */
 
-#ifdef	DOS
-extern char writebuf[BUFSIZ];
-#endif
-
 int clipped = 0;		/* Volume change clipping errors */
 
 static LONG ibufl[BUFSIZ/2];	/* Left/right interleave buffers */
@@ -90,9 +87,6 @@ void process();
 void statistics();
 LONG volumechange();
 void checkeffect(P1(eff_t));
-#ifdef NEED_GETOPT
-int getopt(P3(int,char **,char *));
-#endif
 int flow_effect(P1(int));
 int drain_effect(P1(int));
 
@@ -162,30 +156,27 @@ char **argv;
 	     *	2) fail if the old file already exists - csh mode
 	     */
 	    if (! strcmp(ofile, "-"))
+	    {
 		ft->fp = stdout;
+
+		/* stdout tends to be line-buffered.  Override this */
+		/* to be Full Buffering. */
+		if (setvbuf (ft->fp,NULL,_IOFBF,sizeof(char)*BUFSIZ))
+		    fail("Can't set write buffer");
+	    }
 	    else {
-#ifdef	unix
-		/*	
-		 * Remove old file if it's a text file, but 
-		 * preserve Unix /dev/sound files.
-		 */
-		if ((ft->fp = fopen(ofile, WRITEBINARY)) && 
-		    (filetype(fileno(ft->fp)) == S_IFREG)) {
-		    fclose(ft->fp);
-		    unlink(ofile);
-		    creat(ofile, 0666);
-		    ft->fp = fopen(ofile, WRITEBINARY);
-		}
-#else
+
 		ft->fp = fopen(ofile, WRITEBINARY);
-#endif
+
 		if (ft->fp == NULL)
 		    fail("Can't open output file '%s': %s", 
 			 ofile, strerror(errno));
-#ifdef	DOS
-		if (setvbuf (ft->fp,writebuf,_IOFBF,sizeof(char)*BUFSIZ))
+
+		/* stdout tends to be line-buffered.  Override this */
+		/* to be Full Buffering. */
+		if (setvbuf (ft->fp,NULL,_IOFBF,sizeof(char)*BUFSIZ))
 		    fail("Can't set write buffer");
-#endif
+
 	    } /* end of else != stdout */
 	    
 	    /* Move passed filename */
@@ -207,17 +198,12 @@ char **argv;
 	if (volume <= 0.0)
 		fail("Volume must be greater than 0.0");
 	
-#if	defined(unix) || defined(AMIGA)
-	informat.seekable  = (filetype(fileno(informat.fp)) == S_IFREG);
-	outformat.seekable = (filetype(fileno(outformat.fp)) == S_IFREG); 
-#else
-#if	defined(DOS) || defined(__OS2__)
-	informat.seekable  = 1;
-	outformat.seekable = 1;
-#else
+#if	defined(DUMB_FILESYSETM)
 	informat.seekable  = 0;
 	outformat.seekable = 0;
-#endif
+#else
+	informat.seekable  = (filetype(fileno(informat.fp)) == S_IFREG);
+	outformat.seekable = (filetype(fileno(outformat.fp)) == S_IFREG); 
 #endif
 
 	/* If file types have not been set with -t, set from file names. */
@@ -250,6 +236,12 @@ char **argv;
 	return(0);
 }
 
+#ifdef HAVE_GETOPT_H
+char *getoptstr = "+r:v:t:c:phsuUAagbwlfdDxV";
+#else
+char *getoptstr = "r:v:t:c:phsuUAagbwlfdDxV";
+#endif
+
 void doopts(argc, argv)
 int argc;
 char **argv;
@@ -257,7 +249,7 @@ char **argv;
 	int c;
 	char *str;
 
-	while ((c = getopt(argc, argv, "r:v:t:c:phsuUAagbwlfdDxV")) != -1) {
+	while ((c = getopt(argc, argv, getoptstr)) != -1) {
 		switch(c) {
 		case 'p':
 			soxpreview++;
@@ -777,7 +769,7 @@ eff_t effp;
 			    efftabR[1].name = effp->name;
 		    efftab[1].outinfo.rate = informat.info.rate;
 		    efftab[1].outinfo.channels = informat.info.channels;
-		    efftab[2].name = "rate";
+		    efftab[2].name = "resample";
 		    efftab[3].name = "avg";
 		} else {
 		    neffects = 3;
@@ -792,7 +784,7 @@ eff_t effp;
 		    neffects = 3;
 		    efftab[1].name = effp->name;
 		    efftab[1].outinfo.rate = informat.info.rate;
-		    efftab[2].name = "rate";
+		    efftab[2].name = "resample";
 		    if (informat.info.channels == 2)
 			    efftabR[2].name = "rate";
 		} else {
@@ -861,7 +853,6 @@ LONG y;
 	return y1;
 }
 
-#if	defined(unix) || defined(AMIGA)
 int filetype(fd)
 int fd;
 {
@@ -871,7 +862,6 @@ int fd;
 
 	return st.st_mode & S_IFMT;
 }
-#endif
 
 char *usagestr = 
 "[ gopts ] [ fopts ] ifile [ fopts ] ofile [ effect [ effopts ] ]";
@@ -881,10 +871,7 @@ char *opt;
 {
     int i;
     
-#ifndef	DOS
-	/* single-threaded machines don't really need this */
 	fprintf(stderr, "%s: ", myname);
-#endif
 	if (verbose || !opt)
 		fprintf(stderr, "%s\n\n", version());
 	fprintf(stderr, "Usage: %s\n\n", usagestr);
@@ -917,10 +904,8 @@ void cleanup() {
 	if (outformat.fp) {
 		fclose(outformat.fp);
 		/* remove the output file because we failed, if it's ours. */
-		/* unless its a unix /dev/sound type file */
-#ifdef unix
+		/* Don't if its not a regular file. */
 		if (filetype(fileno(outformat.fp)) == S_IFREG)
-#endif
 		    REMOVE(outformat.filename);
 	}
 }
