@@ -51,10 +51,23 @@
  * User options should override file header - we assumed user knows what
  * they are doing if they specify options.
  * Enhancements and clean up by Graeme W. Gill, 93/5/17
+ *
+ * Info for format tags can be found at:
+ *   http://www.microsoft.com/asf/resources/draft-ietf-fleischman-codec-subtree-01.txt
+ *
  */
 
 #include <string.h>		/* Included for strncmp */
 #include <stdlib.h>		/* Included for malloc and free */
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+#include <stdio.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>		/* For SEEK_* defines if not found in stdio */
+#endif
+
 #include "st.h"
 #include "wav.h"
 
@@ -102,14 +115,14 @@ static LONG gaiCoef2[] = { 0, -256,  0,  64,   0,-208, -232};
  * Lookup tables for IMA ADPCM format
  *
  */
-static const int imaIndexAdjustTable[16] = {
+static int imaIndexAdjustTable[16] = {
    -1, -1, -1, -1,  /* +0 - +3, decrease the step size */
     2, 4, 6, 8,     /* +4 - +7, increase the step size */
    -1, -1, -1, -1,  /* -0 - -3, decrease the step size */
     2, 4, 6, 8,     /* -4 - -7, increase the step size */
 };
 
-static const int imaStepSizeTable[89] = {
+static int imaStepSizeTable[89] = {
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 31, 34,
    37, 41, 45, 50, 55, 60, 66, 73, 80, 88, 97, 107, 118, 130, 143,
    157, 173, 190, 209, 230, 253, 279, 307, 337, 371, 408, 449, 494,
@@ -130,7 +143,10 @@ static const int imaStepSizeTable[89] = {
  *
  */
 
-short ImaAdpcmDecode(unsigned char deltaCode, ImaState_t *state) {
+short ImaAdpcmDecode(deltaCode, state) 
+unsigned char deltaCode;
+ImaState_t *state;
+{
     /* Get the current step size */
    int step;
    int difference;
@@ -211,8 +227,8 @@ ft_t ft;
        
 	/* Got this from xanim */
 
-	state[ch].previousValue = ((signed short)bytePtr[1]<<8) +
-	    (signed short)bytePtr[0];
+	state[ch].previousValue = ((int)bytePtr[1]<<8) +
+	    (int)bytePtr[0];
 	if (state[ch].previousValue & 0x8000)
 	    state[ch].previousValue -= 0x10000;
 
@@ -271,7 +287,10 @@ ft_t ft;
  *
  */
 
-LONG MsAdpcmDecode(LONG deltaCode, MsState_t *state) {
+LONG MsAdpcmDecode(deltaCode, state) 
+LONG deltaCode;
+MsState_t *state;
+{
     LONG predict;
     LONG sample;
     LONG idelta;
@@ -446,8 +465,11 @@ ft_t ft;
     ULONG    data_length;	    /* length of sound data in bytes */
     ULONG    bytespersample;	    /* bytes per sample (per channel */
 
+    /* This is needed for rawread() */
+    rawstartread(ft);
+
     endptr = (char *) &littlendian;
-    if (!*endptr) ft->swap = 1;
+    if (!*endptr) ft->swap = ft->swap ? 0 : 1;
 
     /* If you need to seek around the input file. */
     if (0 && ! ft->seekable)
@@ -456,7 +478,7 @@ ft_t ft;
     if ( fread(magic, 1, 4, ft->fp) != 4 || strncmp("RIFF", magic, 4))
 	fail("WAVE: RIFF header not found");
 
-    len = rllong(ft);
+    len = rlong(ft);
 
     if ( fread(magic, 1, 4, ft->fp) != 4 || strncmp("WAVE", magic, 4))
 	fail("WAVE header not found");
@@ -466,7 +488,7 @@ ft_t ft;
     {
 	if ( fread(magic, 1, 4, ft->fp) != 4 )
 	    fail("WAVE file missing fmt spec");
-	len = rllong(ft);
+	len = rlong(ft);
 	if (strncmp("fmt ", magic, 4) == 0)
 	    break;				/* Found the format chunk */
 
@@ -481,7 +503,7 @@ ft_t ft;
     if ( len < 16 )
 	fail("WAVE file fmt chunk is too short");
 
-    wav->formatTag = rlshort(ft);
+    wav->formatTag = rshort(ft);
     len -= 2;
     switch (wav->formatTag)
     {
@@ -489,6 +511,7 @@ ft_t ft;
 	fail("WAVE file is in unsupported Microsoft Official Unknown format.");
 	
     case WAVE_FORMAT_PCM:
+        /* Default (-1) depends on sample size.  Set that later on. */
 	if (ft->info.style != -1 && ft->info.style != UNSIGNED &&
 	    ft->info.style != SIGN2)
 	    warn("User options overriding style read in .wav header");
@@ -501,15 +524,18 @@ ft_t ft;
 	else
 	    warn("User options overriding style read in .wav header");
 	break;
+
+    case WAVE_FORMAT_IEEE_FLOAT:
+	fail("Sorry, this WAV file is in IEEE Float format.");
 	
-    case WAVE_FORMAT_ALAW:	/* Think I can handle this */
+    case WAVE_FORMAT_ALAW:
 	if (ft->info.style == -1 || ft->info.style == ALAW)
 	    ft->info.style = ALAW;
 	else
 	    warn("User options overriding style read in .wav header");
 	break;
 	
-    case WAVE_FORMAT_MULAW:	/* Think I can handle this */
+    case WAVE_FORMAT_MULAW:
 	if (ft->info.style == -1 || ft->info.style == ULAW)
 	    ft->info.style = ULAW;
 	else
@@ -522,16 +548,30 @@ ft_t ft;
 	fail("Sorry, this WAV file is in Digistd format.");
     case WAVE_FORMAT_DIGIFIX:
 	fail("Sorry, this WAV file is in Digifix format.");
-    case IBM_FORMAT_MULAW:
-	fail("Sorry, this WAV file is in IBM U-law format.");
-    case IBM_FORMAT_ALAW:
-	fail("Sorry, this WAV file is in IBM A-law format.");
-    case IBM_FORMAT_ADPCM:
-	fail("Sorry, this WAV file is in IBM ADPCM format.");
-    default:	fail("WAV file has unknown format type");
+    case WAVE_FORMAT_DOLBY_AC2:
+	fail("Sorry, this WAV file is in Dolby AC2 format.");
+    case WAVE_FORMAT_GSM610:
+	fail("Sorry, this WAV file is in GSM 6.10 format.");
+    case WAVE_FORMAT_ROCKWELL_ADPCM:
+	fail("Sorry, this WAV file is in Rockwell ADPCM format.");
+    case WAVE_FORMAT_ROCKWELL_DIGITALK:
+	fail("Sorry, this WAV file is in Rockwell DIGITALK format.");
+    case WAVE_FORMAT_G721_ADPCM:
+	fail("Sorry, this WAV file is in G.721 ADPCM format.");
+    case WAVE_FORMAT_G728_CELP:
+	fail("Sorry, this WAV file is in G.728 CELP format.");
+    case WAVE_FORMAT_MPEG:
+	fail("Sorry, this WAV file is in MPEG format.");
+    case WAVE_FORMAT_MPEGLAYER3:
+	fail("Sorry, this WAV file is in MPEG Layer 3 format.");
+    case WAVE_FORMAT_G726_ADPCM:
+	fail("Sorry, this WAV file is in G.726 ADPCM format.");
+    case WAVE_FORMAT_G722_ADPCM:
+	fail("Sorry, this WAV file is in G.722 ADPCM format.");
+    default:	fail("WAV file has unknown format type of %x",wav->formatTag);
     }
 
-    wChannels = rlshort(ft);
+    wChannels = rshort(ft);
     len -= 2;
     /* User options take precedence */
     if (ft->info.channels == -1 || ft->info.channels == wChannels)
@@ -539,19 +579,19 @@ ft_t ft;
     else
 	warn("User options overriding channels read in .wav header");
 	
-    wSamplesPerSecond = rllong(ft);
+    wSamplesPerSecond = rlong(ft);
     len -= 4;
     if (ft->info.rate == 0 || ft->info.rate == wSamplesPerSecond)
 	ft->info.rate = wSamplesPerSecond;
     else
 	warn("User options overriding rate read in .wav header");
     
-    wAvgBytesPerSec = rllong(ft);	/* Average bytes/second */
-    wav->blockAlign = rlshort(ft);	/* Block align */
+    wAvgBytesPerSec = rlong(ft);	/* Average bytes/second */
+    wav->blockAlign = rshort(ft);	/* Block align */
     len -= 6;
 
     /* bits per sample per channel */	
-    wBitsPerSample =  rlshort(ft);
+    wBitsPerSample =  rshort(ft);
     len -= 2;
 
     /* ADPCM formats have extended fmt chunk.  Check for those cases. */
@@ -560,10 +600,10 @@ ft_t ft;
 	if (wBitsPerSample != 4)
 	    fail("Can only handle 4-bit MS ADPCM in wav files");
 
-	wExtSize = rlshort(ft);
-	wav->samplesPerBlock = rlshort(ft);
+	wExtSize = rshort(ft);
+	wav->samplesPerBlock = rshort(ft);
 	wav->bytesPerBlock = (wav->samplesPerBlock + 7)/2 * ft->info.channels;
-	wNumCoefs = rlshort(ft);
+	wNumCoefs = rshort(ft);
 	wav->packet = (unsigned char *)malloc(wav->blockAlign);
 	len -= 6;
 	    
@@ -571,14 +611,16 @@ ft_t ft;
 	/* Use ft->info.channels after this becuase wChannels is now bad */
 	while (wChannels-- > 0)
 	    wav->samples[wChannels] = (short *)malloc(wav->samplesPerBlock*sizeof(short));
+	/* Here we are setting the bytespersample AFTER de-compression */
+	bytespersample = WORD;
     }
     else if (wav->formatTag == WAVE_FORMAT_IMA_ADPCM)
     {
 	if (wBitsPerSample != 4)
 	    fail("Can only handle 4-bit IMA ADPCM in wav files");
 
-	wExtSize = rlshort(ft);
-	wav->samplesPerBlock = rlshort(ft);
+	wExtSize = rshort(ft);
+	wav->samplesPerBlock = rshort(ft);
 	wav->bytesPerBlock = (wav->samplesPerBlock + 7)/2 * ft->info.channels;
 	wav->packet = (unsigned char *)malloc(wav->blockAlign);
 	len -= 4;
@@ -587,9 +629,14 @@ ft_t ft;
 	/* Use ft->info.channels after this becuase wChannels is now bad */
 	while (wChannels-- > 0)
 	    wav->samples[wChannels] = (short *)malloc(wav->samplesPerBlock*sizeof(short));
+	/* Here we are setting the bytespersample AFTER de-compression */
+	bytespersample = WORD;
+    }
+    else
+    {
+      bytespersample = (wBitsPerSample + 7)/8;
     }
 
-    bytespersample = (wBitsPerSample + 7)/8;
     switch (bytespersample)
     {
 	
@@ -599,11 +646,10 @@ ft_t ft;
 	    ft->info.size = BYTE;
 	else
 	    warn("User options overriding size read in .wav header");
-	if (ft->info.style == -1 || ft->info.style == UNSIGNED)
+
+	/* Now we have enough information to set default styles. */
+	if (ft->info.style == -1)
 	    ft->info.style = UNSIGNED;
-	else if (ft->info.style != ALAW && ft->info.style != ULAW &&
-		 ft->info.style != ADPCM) 
-	    warn("User options overriding style read in .wav header");
 	break;
 	
     case WORD:
@@ -611,10 +657,10 @@ ft_t ft;
 	    ft->info.size = WORD;
 	else
 	    warn("User options overriding size read in .wav header");
-	if (ft->info.style == -1 || ft->info.style == SIGN2)
+
+	/* Now we have enough information to set default styles. */
+	if (ft->info.style == -1)
 	    ft->info.style = SIGN2;
-	else
-	    warn("User options overriding style read in .wav header");
 	break;
 	
     case DWORD:
@@ -622,10 +668,10 @@ ft_t ft;
 	    ft->info.size = DWORD;
 	else
 	    warn("User options overriding size read in .wav header");
-	if (ft->info.style == -1 || ft->info.style == SIGN2)
+
+	/* Now we have enough information to set default styles. */
+	if (ft->info.style == -1)
 	    ft->info.style = SIGN2;
-	else
-	    warn("User options overriding style read in .wav header");
 	break;
 	
     default:
@@ -644,7 +690,7 @@ ft_t ft;
     {
 	if ( fread(magic, 1, 4, ft->fp) != 4 )
 	    fail("WAVE file has missing data chunk");
-	len = rllong(ft);
+	len = rlong(ft);
 	if (strncmp("data", magic, 4) == 0)
 	    break;				/* Found the data chunk */
 	
@@ -783,6 +829,9 @@ ft_t ft;
     if (wav->packet) free(wav->packet);
     if (wav->samples[0]) free(wav->samples[0]);
     if (wav->samples[1]) free(wav->samples[1]);
+
+    /* Needed for rawread() */
+    rawstopread(ft);
 }
 
 void wavstartwrite(ft) 
@@ -793,7 +842,7 @@ ft_t ft;
 	char	*endptr;
 
 	endptr = (char *) &littlendian;
-	if (!*endptr) ft->swap = 1;
+	if (!*endptr) ft->swap = ft->swap ? 0 : 1;
 
 	wav->numSamples = 0;
 	wav->second_header = 0;
@@ -817,43 +866,38 @@ ft_t ft;
         ULONG  data_length;             	/* length of sound data in bytes */
 	ULONG  bytespersample; 			/* bytes per sample (per channel) */
 
+	/* Needed for rawwrite() */
+	rawstartwrite(ft);
+
 	switch (ft->info.size)
 	{
 		case BYTE:
-			if (ft->info.style == -1 || ft->info.style == UNSIGNED)
-				ft->info.style = UNSIGNED;
-			else if (!wav->second_header && ft->info.style != ALAW && ft->info.style != ULAW && ft->info.style != ADPCM) 
-				warn("User options overriding style written to .wav header");
-			if (ft->info.style == ADPCM)
+		        wBitsPerSample = 8;
+			if (ft->info.style != UNSIGNED &&
+			    ft->info.style != ULAW &&
+			    ft->info.style != ALAW &&
+			    !wav->second_header)
 			{
-			    warn("Can not support writing ADPCM style. Overriding to UNSIGNED\n");
-			    ft->info.style = UNSIGNED;
-			    wBitsPerSample = 8;
-			    /* wBitsPerSample = 4; */
+				warn("Only support unsigned, ulaw, or alaw with 8-bit data.  Forcing to unsigned");
+				ft->info.style = UNSIGNED;
 			}
-			else
-				wBitsPerSample = 8;
 			break;
 		case WORD:
 			wBitsPerSample = 16;
-			if (ft->info.style == -1 || ft->info.style == SIGN2)
+			if ((ft->info.style == UNSIGNED ||
+			     ft->info.style == ULAW ||
+			     ft->info.style == ALAW) &&
+			    !wav->second_header)
+			{
+				warn("Do not support Unsigned, ulaw, or alay with 16 bit data.  Forcing to Signed");
 				ft->info.style = SIGN2;
-			else if (!wav->second_header)
-				warn("User options overriding style written to .wav header");
+			}
 			break;
 		case DWORD:
 			wBitsPerSample = 32;
-			if (ft->info.style == -1 || ft->info.style == SIGN2)
-				ft->info.style = SIGN2; 
-			else if (!wav->second_header)
-				warn("User options overriding style written to .wav header");
 			break;
 		default:
 			wBitsPerSample = 32;
-			if (ft->info.style == -1)
-				ft->info.style = SIGN2; 
-			if (!wav->second_header)
-				warn("Warning - writing bad .wav file using %s",sizes[ft->info.size]);
 			break;
 	}
 
@@ -861,29 +905,25 @@ ft_t ft;
 	{
 		case UNSIGNED:
 			wFormatTag = WAVE_FORMAT_PCM;
-			if (wBitsPerSample != 8 && !wav->second_header)
-				warn("Warning - writing bad .wav file using unsigned data and %d bits/sample",wBitsPerSample);
 			break;
 		case SIGN2:
 			wFormatTag = WAVE_FORMAT_PCM;
-			if (wBitsPerSample == 8 && !wav->second_header)
-				warn("Warning - writing bad .wav file using signed data and %d bits/sample",wBitsPerSample);
 			break;
 		case ALAW:
 			wFormatTag = WAVE_FORMAT_ALAW;
-			if (wBitsPerSample != 8 && !wav->second_header)
-				warn("Warning - writing bad .wav file using A-law data and %d bits/sample",wBitsPerSample);
 			break;
 		case ULAW:
 			wFormatTag = WAVE_FORMAT_MULAW;
-			if (wBitsPerSample != 8 && !wav->second_header)
-				warn("Warning - writing bad .wav file using U-law data and %d bits/sample",wBitsPerSample);
 			break;
 		case ADPCM:
-			wFormatTag = WAVE_FORMAT_IMA_ADPCM;
+			wFormatTag = WAVE_FORMAT_PCM;
+		        warn("Can not support writing ADPCM style. Overriding to Signed Words\n");
+			ft->info.style = SIGN2;
+			wBitsPerSample = 16;
+			/* wFormatTag = WAVE_FORMAT_IMA_ADPCM;
+			   wBitsPerSample = 4;
 			if (wBitsPerSample != 4 && !wav->second_header)
-			warn("Warning - writing bad .wav file using IMA ADPCM and %d bits/sample",wBitsPerSample);
-			break;
+			break; */
 	}
 	
 	
@@ -904,19 +944,19 @@ ft_t ft;
 
 	/* figured out header info, so write it */
 	fputs("RIFF", ft->fp);
-	wllong(ft, data_length + 8+16+12);	/* Waveform chunk size: FIXUP(4) */
+	wlong(ft, data_length + 8+16+12);	/* Waveform chunk size: FIXUP(4) */
 	fputs("WAVE", ft->fp);
 	fputs("fmt ", ft->fp);
-	wllong(ft, (LONG)16);		/* fmt chunk size */
-	wlshort(ft, wFormatTag);
-	wlshort(ft, wChannels);
-	wllong(ft, wSamplesPerSecond);
-	wllong(ft, wAvgBytesPerSec);
-	wlshort(ft, wBlockAlign);
-	wlshort(ft, wBitsPerSample);
+	wlong(ft, (LONG)16);		/* fmt chunk size */
+	wshort(ft, wFormatTag);
+	wshort(ft, wChannels);
+	wlong(ft, wSamplesPerSecond);
+	wlong(ft, wAvgBytesPerSec);
+	wshort(ft, wBlockAlign);
+	wshort(ft, wBitsPerSample);
 	
 	fputs("data", ft->fp);
-	wllong(ft, data_length);		/* data chunk size: FIXUP(40) */
+	wlong(ft, data_length);		/* data chunk size: FIXUP(40) */
 
 	if (!wav->second_header) {
 		report("Writing Wave file: %s format, %d channel%s, %d samp/sec",
@@ -942,12 +982,15 @@ void
 wavstopwrite(ft) 
 ft_t ft;
 {
+	/* Call this to flush out any remaining data. */
+	rawstopwrite(ft);
+
 	/* All samples are already written out. */
 	/* If file header needs fixing up, for example it needs the */
  	/* the number of samples in a field, seek back and write them here. */
 	if (!ft->seekable)
 		return;
-	if (fseek(ft->fp, 0L, 0) != 0)
+	if (fseek(ft->fp, 0L, SEEK_SET) != 0)
 		fail("Sorry, can't rewind output file to rewrite .wav header.");
 	((wav_t) ft->priv)->second_header = 1;
 	wavwritehdr(ft);
@@ -968,6 +1011,8 @@ unsigned wFormatTag;
 			return "Microsoft PCM";
 		case WAVE_FORMAT_ADPCM:
 			return "Microsoft ADPCM";
+	        case WAVE_FORMAT_IEEE_FLOAT:
+		       return "IEEE Float";
 		case WAVE_FORMAT_ALAW:
 			return "Microsoft A-law";
 		case WAVE_FORMAT_MULAW:
@@ -980,12 +1025,26 @@ unsigned wFormatTag;
 			return "Digistd format.";
 		case WAVE_FORMAT_DIGIFIX:
 			return "Digifix format.";
-		case IBM_FORMAT_MULAW:
-			return "IBM U-law format.";
-		case IBM_FORMAT_ALAW:
-			return "IBM A-law";
-                case IBM_FORMAT_ADPCM:
-                	return "IBM ADPCM";
+		case WAVE_FORMAT_DOLBY_AC2:
+			return "Dolby AC2";
+		case WAVE_FORMAT_GSM610:
+			return "GSM 6.10";
+		case WAVE_FORMAT_ROCKWELL_ADPCM:
+			return "Rockwell ADPCM";
+		case WAVE_FORMAT_ROCKWELL_DIGITALK:
+			return "Rockwell DIGITALK";
+		case WAVE_FORMAT_G721_ADPCM:
+			return "G.721 ADPCM";
+		case WAVE_FORMAT_G728_CELP:
+			return "G.728 CELP";
+		case WAVE_FORMAT_MPEG:
+			return "MPEG";
+		case WAVE_FORMAT_MPEGLAYER3:
+			return "MPEG Layer 3";
+		case WAVE_FORMAT_G726_ADPCM:
+			return "G.726 ADPCM";
+		case WAVE_FORMAT_G722_ADPCM:
+			return "G.722 ADPCM";
 		default:
 			return "Unknown";
 	}
